---------------------------------------------------------------------------------------------
-- QHParty Class
---------------------------------------------------------------------------------------------
QHParty = {};
QHParty.__index = QHParty;

function QHParty:new(id)

	-- Metatable
	local self = {};
     	setmetatable(self, QHParty);
     	
     	-- Position Variables
	self.x = 0;
	self.y = 0;
	self.align = "TOPLEFT";
	self.relative = "TOPLEFT";
	self.anchor = UIParent;
	self.width = 0;
	self.height = 0;
	self.moving = nil;
	
	-- Frame Variables
	self.id = id;
	self.name = QH:Concat("QHParty", id);
	self.frame = CreateFrame("Frame", self.name, UIParent);
	
	-- Unit Frame Anchors
	self.playeranchor = QH:CloneFrame(PlayerFrame, QH:Concat(self.name, "_Anchor_player"));
	self.petanchor = QH:CloneFrame(PetFrame, QH:Concat(self.name, "_Anchor_pet"));
	self.party1anchor = QH:CloneFrame(PartyMemberFrame1, QH:Concat(self.name, "_Anchor_party1"));
	self.partypet1anchor = QH:CloneFrame(PartyMemberFrame1PetFrame, QH:Concat(self.name, "_Anchor_partypet1"));
	self.party2anchor = QH:CloneFrame(PartyMemberFrame2, QH:Concat(self.name, "_Anchor_party2"));
	self.partypet2anchor = QH:CloneFrame(PartyMemberFrame2PetFrame, QH:Concat(self.name, "_Anchor_partypet2"));
	self.party3anchor = QH:CloneFrame(PartyMemberFrame3, QH:Concat(self.name, "_Anchor_party3"));
	self.partypet3anchor = QH:CloneFrame(PartyMemberFrame3PetFrame, QH:Concat(self.name, "_Anchor_partypet3"));
	self.party4anchor = QH:CloneFrame(PartyMemberFrame4, QH:Concat(self.name, "_Anchor_party4"));
	self.partypet4anchor = QH:CloneFrame(PartyMemberFrame4PetFrame, QH:Concat(self.name, "_Anchor_partypet4"));
	
	-- Unit Frames
	self.player = QHUnit:new("player");
	self.player:SetPosition(3, 5, "LEFT", "RIGHT", self.playeranchor);
	self.pet = QHUnit:new("pet");
	self.pet:SetPosition(0, 4, "LEFT", "RIGHT", self.petanchor);
	self.party1 = QHUnit:new("party1");
	self.party1:SetPosition(3, 5, "LEFT", "RIGHT", self.party1anchor);
	self.partypet1 = QHUnit:new("partypet1");
	self.partypet1:SetPosition(0, 4, "LEFT", "RIGHT", self.partypet1anchor);
	self.party2 = QHUnit:new("party2");
	self.party2:SetPosition(3, 5, "LEFT", "RIGHT", self.party2anchor);
	self.partypet2 = QHUnit:new("partypet2");
	self.partypet2:SetPosition(0, 4, "LEFT", "RIGHT", self.partypet2anchor);
	self.party3 = QHUnit:new("party3");
	self.party3:SetPosition(3, 5, "LEFT", "RIGHT", self.party3anchor);
	self.partypet3 = QHUnit:new("partypet3");
	self.partypet3:SetPosition(0, 4, "LEFT", "RIGHT", self.partypet3anchor);
	self.party4 = QHUnit:new("party4");
	self.party4:SetPosition(3, 5, "LEFT", "RIGHT", self.party4anchor);
	self.partypet4 = QHUnit:new("partypet4");
	self.partypet4:SetPosition(0, 4, "LEFT", "RIGHT", self.partypet4anchor);
	
	-- Target Frame 
	-- TODO: Create Target Class
	self.targetpoint, self.targetrelative, self.targetalign, self.targetx, self.targety = getglobal("TargetFrame"):GetPoint();
	
	-- Reference Variables
	self.obj = self;
	self.parent = nil;
	self.frame.obj = self;
	
	-- Call Update Function and Return
	self:Update();
	return self;
end

---------------------------------------------------------------------------------------------
-- QHParty Methods
---------------------------------------------------------------------------------------------
function QHParty:Initialize()
	
end

---------------------------------------------------------------------------------------------
-- QHParty Events
---------------------------------------------------------------------------------------------
function QHParty:NewUpdate()

	-- Secure Code
	if not InCombatLockdown() then

		self:UpdateAnchors();
				
		-- Adjust Target Frame
		-- TODO: Create Target Class
		getglobal("TargetFrame"):SetClampedToScreen(true);
		if QH:GetSetting("TargetFrame_Disable_Adjust") then
			getglobal("TargetFrame"):SetUserPlaced(false);
			getglobal("TargetFrame"):SetPoint(self.targetpoint, self.targetrelative, self.targetalign, self.targetx, self.targety);
		else
			getglobal("TargetFrame"):RegisterForDrag("LeftButton");
			getglobal("TargetFrame"):SetScript("OnDragStart", function() if(IsShiftKeyDown()) then getglobal("TargetFrame"):StartMoving(); end end);
			getglobal("TargetFrame"):SetScript("OnDragStop", function() getglobal("TargetFrame"):StopMovingOrSizing(); getglobal("TargetFrame"):SetUserPlaced(true); end);
			if not getglobal("TargetFrame"):IsUserPlaced() then
				getglobal("TargetFrame"):SetPoint(self.targetpoint, self.targetrelative, self.targetalign, self.targetx + self.player.width, self.targety);
			end
		end
	end
	
	-- Show/Hide Party Bars
	self.player:Show();
	self.pet:Show();
	local hide_party = QH:GetSetting("Raid_Party_Hide");
	if hide_party and (UnitInRaid("player") or GetNumRaidMembers() > 0) then
		self.party1:Hide();
		self.partypet1:Hide();
		self.party2:Hide();
		self.partypet2:Hide();
		self.party3:Hide();
		self.partypet3:Hide();
		self.party4:Hide();
		self.partypet4:Hide();
	else
		self.party1:Show();
		self.partypet1:Show();
		self.party2:Show();
		self.partypet2:Show();
		self.party3:Show();
		self.partypet3:Show();
		self.party4:Show();
		self.partypet4:Show();
	end
	
	-- Update Range
	self.party1:UpdateRange();
	self.partypet1:UpdateRange();
	self.party2:UpdateRange();
	self.partypet2:UpdateRange();
	self.party3:UpdateRange();
	self.partypet3:UpdateRange();
	self.party4:UpdateRange();
	self.partypet4:UpdateRange();
end

function QHParty:Update()

	-- Update Anchors
	if not InCombatLockdown() then
	
		self:UpdateAnchors();
		
		-- Adjust Target Frame 
		-- TODO: Create Target Class
		getglobal("TargetFrame"):SetClampedToScreen(true);
		if QH:GetSetting("TargetFrame_Disable_Adjust") then
			getglobal("TargetFrame"):SetUserPlaced(false);
			getglobal("TargetFrame"):SetPoint(self.targetpoint, self.targetrelative, self.targetalign, self.targetx, self.targety);
		else
			getglobal("TargetFrame"):RegisterForDrag("LeftButton");
			getglobal("TargetFrame"):SetScript("OnDragStart", function() if(IsShiftKeyDown()) then getglobal("TargetFrame"):StartMoving(); end end);
			getglobal("TargetFrame"):SetScript("OnDragStop", function() getglobal("TargetFrame"):StopMovingOrSizing(); getglobal("TargetFrame"):SetUserPlaced(true); end);
			if not getglobal("TargetFrame"):IsUserPlaced() then
				getglobal("TargetFrame"):SetPoint(self.targetpoint, self.targetrelative, self.targetalign, self.targetx + self.player.width, self.targety);
			end
		end
	end

	-- Update Player Bar
	self.player:Show();
	self.pet:Show();
	self.player:Update();
	self.pet:Update();

	-- Update Party Bars
	local hide_party = QH:GetSetting("Raid_Party_Hide");
	if hide_party and (UnitInRaid("player") or GetNumRaidMembers() > 0) then
		self.party1:Hide();
		self.partypet1:Hide();
		self.party2:Hide();
		self.partypet2:Hide();
		self.party3:Hide();
		self.partypet3:Hide();
		self.party4:Hide();
		self.partypet4:Hide();
	else
		self.party1:Show();
		self.partypet1:Show();
		self.party2:Show();
		self.partypet2:Show();
		self.party3:Show();
		self.partypet3:Show();
		self.party4:Show();
		self.partypet4:Show();
		self.party1:Update();
		self.partypet1:Update();
		self.party2:Update();
		self.partypet2:Update();
		self.party3:Update();
		self.partypet3:Update();
		self.party4:Update();
		self.partypet4:Update();
	end
end

function QHParty:UpdateAnchors()

	QH:CopyFrame(PlayerFrame, self.playeranchor);
	QH:CopyFrame(PetFrame, self.petanchor);
	QH:CopyFrame(PartyMemberFrame1, self.party1anchor);
	QH:CopyFrame(PartyMemberFrame1PetFrame, self.partypet1anchor);
	QH:CopyFrame(PartyMemberFrame2, self.party2anchor);
	QH:CopyFrame(PartyMemberFrame2PetFrame, self.partypet2anchor);
	QH:CopyFrame(PartyMemberFrame3, self.party3anchor);
	QH:CopyFrame(PartyMemberFrame3PetFrame, self.partypet3anchor);
	QH:CopyFrame(PartyMemberFrame4, self.party4anchor);
	QH:CopyFrame(PartyMemberFrame4PetFrame, self.partypet4anchor);
end